function cpoint

% *************************************************************
% VOT: Set random seed to a different value every time.
% *************************************************************

RandStream.setGlobalStream(RandStream('mt19937ar', 'Seed', sum(clock)));

% **********************************
% VOT: Get initialization data
% **********************************
[handle, image, region] = vot('rectangle');

image = imread(image); % Read first image from file
% TODO: Initialize the tracker with first image and the given initialization region


global pastImages
imageWidth = size(image,2);
imageHeight = size(image,1);
[bbox,imSize] = cpoint_initialize(image, region);

frameT = imcrop(image, bbox);% Initial template
%bboxB is twise as the 1st template. frameB - template with border
borderB = round(max(imSize)*0.5);    
bboxB = [(bbox(1) - borderB),(bbox(2) - borderB),(bbox(3) + (2*borderB)),(bbox(4) + (2*borderB))];
frameB = imcrop(image, bboxB);

oldFrameT = frameT; % For histTracker
oldFrameB = frameB;

% Initialize pointtracker
oldBboxPT = bbox;
oldPoints = detectMinEigenFeatures(rgb2gray(image),'ROI',oldBboxPT,'FilterSize',3);
% oldPoints = detectHarrisFeatures(rgb2gray(oldFrameB),'ROI',oldBbox);
tracker = vision.PointTracker('MaxBidirectionalError',1);
initialize(tracker,oldPoints.Location,image);


oldBbox = bbox; % Template cordinates in image
oldBbox(1:2) = bbox(1:2)-bboxB(1:2); % Changed to template cordinates in frameB

template = rgb2gray(frameT);
pastImages = struct('template', template, 'size', [bbox(3), bbox(4)]);


while true

    % **********************************
    % VOT: Get next frame
    % **********************************
    [handle, image] = handle.frame(handle);

    if isempty(image)
        break;
    end

    image = imread(image); % Read the image from file
    % TODO: Perform a tracking step with the image, obtain new region

    
    %bboxB is twise as the 1st template. frameB - template with borders
    borderB = round(max(bbox(3),bbox(4))*0.5);    
    bboxB = [(bbox(1) - borderB),(bbox(2) - borderB),(bbox(3) + (2*borderB)),(bbox(4) + (2*borderB))];
    bboxB = adjustBbox(bboxB,imageWidth,imageHeight); % change bbox for out of image cordinates
    frameB = imcrop(image, bboxB);

    count = 1;
    bboxC =[];
    score = 0;
    bboxPT = [];
    
    [bboxPTN, keypoints, maxCount] = pointTrackerNew(frameB); 
    if size(bboxPTN,2)>1
        bboxPTN(1:2) = bboxPTN(1:2)+bboxB(1:2);
        bboxC(:,:,count) = bboxPTN;
        count = count+1;
        
        if maxCount>10
            template = imcrop(image, bboxPTN);
            template = rgb2gray(template);
            pastImages(2:end+1)=pastImages(1:end); % Update the latest template
            pastImages(1) = struct('template', template, 'size', [bbox(3), bbox(4)]); % Update the latest template
            if size(pastImages,2)>2 % Limit to 2 recent images
                pastImages = pastImages(1:2);
            end
        end
    end
    
    [bboxPT,pointCount] = pointTracker(image,tracker,oldPoints,oldBboxPT);
    if size(bboxPT,2)>1
        bboxC(:,:,count) = bboxPT;
        count = count+1;

        if pointCount>10
            template = imcrop(image, bboxPT);
            template = rgb2gray(template);
            pastImages(2:end+1)=pastImages(1:end); % Update the latest template
            pastImages(1) = struct('template', template, 'size', [bbox(3), bbox(4)]); % Update the latest template
            if size(pastImages,2)>2 % Limit to 2 recent images
                pastImages = pastImages(1:2);
            end
        end
    end         
       
    if size(bboxPTN,2)>1 || size(bboxPT,2)>1
        bbox(1) = mean(bboxC(1,1,:));
        bbox(2) = mean(bboxC(1,2,:));
        bbox(3) = mean(bboxC(1,3,:));
        bbox(4) = mean(bboxC(1,4,:));
        bbox = adjustBbox(bbox,imageWidth,imageHeight);
        
        region = bbox; % Average bbox from point trackers

    else % If both point trackers fail
        oldBbox(1:2) = bbox(1:2)-bboxB(1:2);
        [bboxHT,score] = histTracker(oldBbox,oldFrameB,frameB); % mex file
        if score>0.4
            bboxHT(1:2) = bboxHT(1:2)+bboxB(1:2);
            bbox = bboxHT;
            region = bboxHT; % bboox from histTracker
        end
    end

    oldBbox = bbox; % This is for histTracker
    oldFrameB = frameB;
    
    if size(keypoints,1)>6 || pointCount>5 || score>0.4
        confidence = 1; % Confidence of presence of object
    else
        confidence = 0;
    end
    
    
    region = bbox; % bbox of previous frame
    
   
    % **********************************
    % VOT: Report position for frame
    % **********************************
    handle = handle.report(handle, region, confidence);

end;

% **********************************
% VOT: Output the results
% **********************************
handle.quit(handle);

end