function [bbox, keypoints, maxCount] = pointTrackerNew(sceneImage)
% This function finds the average bounding box using 6 keypoint types.
% 'maxCount' is the highest number of correct keypoints among 4 keypoint
% types.
% 'keypoints' is the sum of correct keypoints (after removing outliers).

global pastImages

if size(sceneImage,3)==3
    sceneImage = rgb2gray(sceneImage);
end

[keypoints,countKAZE,countSURF,countFAST,countBRISK] = extractKeypoints(sceneImage);

% find the max keypoints match.
[maxCountKAZE,idxKAZE(1)] = max(countKAZE);
[maxCountSURF,idxSURF(1)] = max(countSURF);
[maxCountFAST,idxFAST(1)] = max(countFAST);
[maxCountBRISK,idxBRISK(1)] = max(countBRISK);
[maxCount, ~] = max([maxCountKAZE; maxCountSURF; maxCountFAST; maxCountBRISK]);

if maxCount>10
    % find the second max keypoints match.
    if size(pastImages,2)>1
        try
            [maxCountKAZE2,idxKAZE(2)] = max(countKAZE(countKAZE<max(countKAZE)));
        catch % error when [0 0 0 0 0 0] or [x x x x x x]
            maxCountKAZE2 = 0; idxKAZE(2) = 1; % idxKAZE(2) = 1 is not correct value, but doesn't affect the results
        end
        try
            [maxCountSURF2,idxSURF(2)] = max(countSURF(countSURF<max(countSURF)));
        catch
            maxCountSURF2 = 0; idxSURF(2) = 1;
        end
        try
        [maxCountFAST2,idxFAST(2)] = max(countFAST(countFAST<max(countFAST)));
        catch
            maxCountFAST2 = 0; idxFAST(2) =1;
        end
        try
        [maxCountBRISK2,idxBRISK(2)] = max(countBRISK(countBRISK<max(countBRISK)));
        catch
            maxCountBRISK2 = 0; idxBRISK(2) = 1;
        end
        c = 2;
    else
        c = 1;
    end


    parfor i=1:c
        idx = idxKAZE(i);

        scenePoints = detectKAZEFeatures(sceneImage,'Diffusion','region');
        [sceneFeatures, scenePoints] = extractFeatures(sceneImage, scenePoints);
        boxImage = pastImages(idx).template;
        boxPoints = detectKAZEFeatures(boxImage,'Diffusion','region');
        [boxFeatures, boxPoints] = extractFeatures(boxImage, boxPoints);
        boxPairs = matchFeatures(boxFeatures, sceneFeatures, 'MatchThreshold',15);

        if size(boxPairs,1)>4
        bboxKAZE(:,:,i) = pointTrackerNewTYPE(boxPoints,scenePoints,boxPairs,size(boxImage));
        end
        
    end     

    parfor i=1:c
        idx = idxSURF(i);
       
        scenePoints = detectSURFFeatures(sceneImage,'MetricThreshold',1,'NumScaleLevels',6,'NumOctaves',1);
        [sceneFeatures, scenePoints] = extractFeatures(sceneImage, scenePoints);
        boxImage = pastImages(idx).template;
        boxPoints = detectSURFFeatures(boxImage,'MetricThreshold',1,'NumScaleLevels',6,'NumOctaves',1);
        [boxFeatures, boxPoints] = extractFeatures(boxImage, boxPoints);
        boxPairs = matchFeatures(boxFeatures, sceneFeatures, 'MatchThreshold',15);

        if size(boxPairs,1)>4
        bboxSURF(:,:,i) = pointTrackerNewTYPE(boxPoints,scenePoints,boxPairs);
        end
    end     

    parfor i=1:c
        idx = idxFAST(i);

        scenePoints = detectFASTFeatures(sceneImage, 'MinContrast', 0.05, 'MinQuality',0.05);
        [sceneFeatures, scenePoints] = extractFeatures(sceneImage, scenePoints);
        boxImage = pastImages(idx).template;
        boxPoints = detectFASTFeatures(boxImage, 'MinContrast', 0.05, 'MinQuality',0.05);
        [boxFeatures, boxPoints] = extractFeatures(boxImage, boxPoints);
        boxPairs = matchFeatures(boxFeatures, sceneFeatures, 'MatchThreshold',15);

        if size(boxPairs,1)>4
        bboxFAST(:,:,i) = pointTrackerNewTYPE(boxPoints,scenePoints,boxPairs);
        end
    end     

    parfor i=1:c
        idx = idxBRISK(i);

        scenePoints = detectBRISKFeatures(sceneImage, 'MinContrast', 0.05,'NumOctaves',1,'MinQuality',0.05);
        [sceneFeatures, scenePoints] = extractFeatures(sceneImage, scenePoints);
        boxImage = pastImages(idx).template;
        boxPoints = detectBRISKFeatures(boxImage, 'MinContrast', 0.05,'NumOctaves',1,'MinQuality',0.05);
        [boxFeatures, boxPoints] = extractFeatures(boxImage, boxPoints);
        boxPairs = matchFeatures(boxFeatures, sceneFeatures, 'MatchThreshold',15);
   
        if size(boxPairs,1)>4
        bboxBRISK(:,:,i) = pointTrackerNewTYPE(boxPoints,scenePoints,boxPairs);
        end
    end     


% find average bbox, for keypoints>4
        count = 1; %number of valid bbox, >4 keypoints
        if maxCountKAZE>4 && bboxKAZE(:,3,1)~=0 % && bboxKAZE(:,3,1)~=0 for try/catch
            bboxC(:,:,count) = bboxKAZE(:,:,1);
            count = count+1;
        end
        if maxCountSURF>4 && bboxSURF(:,3,1)~=0
            bboxC(:,:,count) = bboxSURF(:,:,1);
            count = count+1;
        end            
        if maxCountFAST>4 && bboxFAST(:,3,1)~=0
            bboxC(:,:,count) = bboxFAST(:,:,1);
            count = count+1;
        end
        if maxCountBRISK>4 && bboxBRISK(:,3,1)~=0
            bboxC(:,:,count) = bboxBRISK(:,:,1);
            count = count+1;
        end
        if size(pastImages,2)>1
            if maxCountKAZE2>4 && bboxKAZE(:,3,1)~=0 && size(bboxKAZE,3)>1
                bboxC(:,:,count) = bboxKAZE(:,:,2);
                count = count+1;
            end
            if maxCountSURF2>4 && bboxSURF(:,3,1)~=0 && size(bboxSURF,3)>1
                bboxC(:,:,count) = bboxSURF(:,:,2);
                count = count+1;
            end            
            if maxCountFAST2>4 && bboxFAST(:,3,1)~=0 && size(bboxFAST,3)>1
                bboxC(:,:,count) = bboxFAST(:,:,2);
                count = count+1;
            end
            if maxCountBRISK2>4 && bboxBRISK(:,3,1)~=0 && size(bboxBRISK,3)>1
                bboxC(:,:,count) = bboxBRISK(:,:,2);
            end
        end
        try
            bbox(1) = mean(bboxC(1,1,:));
            bbox(2) = mean(bboxC(1,2,:));
            bbox(3) = mean(bboxC(1,3,:));
            bbox(4) = mean(bboxC(1,4,:));
        catch
            bbox = nan;
        end

%         template = imcrop(sceneImage, bbox);
%         pastImages(2:end+1)=pastImages(1:end); % Update the latest template
%         pastImages(1) = struct('template', template, 'size', [bbox(3), bbox(4)]); % Update the latest template
%         if size(pastImages,2)>6 % Limit to 10 recent images
%             pastImages = pastImages(1:6);
%         end
else
%     display('cant detect, try next tracker');
    bbox = nan;
end
end
